/*!
        @file    $Id:: field_F_imp.cpp #$

        @brief

        @author  <Hideo Matsufuru> hideo.matsufuru@kek.jp(matsufuru) 
                 $LastChangedBy: matufuru $

        @date    $LastChangedDate:: 2014-06-02 23:54:18 #$

        @version $LastChangedRevision: 1054 $
*/

#include "field_F.h"

using std::valarray;

#include "threadManager_OpenMP.h"

#if defined USE_GROUP_SU3
#include "field_F_imp_SU3.inc"
#elif defined USE_GROUP_SU2
#include "field_F_imp_SU2.inc"
#elif defined USE_GROUP_SU_N
#include "field_F_imp_SU_N.inc"
#endif

//====================================================================
void Field_F::check()
{
  //  assert(NC == CommonParameters::Nc());
  //  assert(ND == CommonParameters::Nd());

  check_Nc();

}

//====================================================================
void Field_F::mult_GM(const GammaMatrix& gm, const Field_F& x)
{
  assert(x.nex() == m_Nex);
  assert(x.nvol() == m_Nvol);

  int    id[ND];
  int    idc_r[ND];
  int    idc_i[ND];
  double gv_r[ND];
  double gv_i[ND];

  for(int s = 0; s < ND; ++s){
    id[s] = gm.index(s);
    gv_r[s] = gm.value_r(s);
    gv_i[s] = gm.value_i(s);
    idc_r[s] = gm.index_c(s);
    idc_i[s] = 1-idc_r[s];
  }

  int nth = ThreadManager_OpenMP::get_num_threads();
  int ith = ThreadManager_OpenMP::get_thread_id();

  int is  = m_Nvol * ith / nth;
  int ns  = m_Nvol * (ith+1) / nth - is;

  double* v = ptr(m_Nin*is);
  double* w = const_cast<Field_F*>(&x)->ptr(m_Nin*is);

  for(int ex = 0; ex < m_Nex; ++ex){
   for(int site = 0; site < ns; ++site){
     int iv = NCD2 * (site + m_Nvol*ex);
    for(int s = 0; s < ND; ++s){
       int iv2 =    s  * NC2 + iv;
       int iw2 = id[s] * NC2 + iv;
     for(int ic = 0; ic < NC; ++ic){
       v[2*ic   + iv2] = gv_r[s] * w[2*ic+idc_r[s]+iw2];
       v[2*ic+1 + iv2] = gv_i[s] * w[2*ic+idc_i[s]+iw2];
     }
    }
   }
  }

}

//====================================================================
void Field_F::mult_iGM(const GammaMatrix& gm, const Field_F& x)
{
  assert(x.nex() == m_Nex);
  assert(x.nvol() == m_Nvol);

  int    id[ND];
  int    idc_r[ND];
  int    idc_i[ND];
  double gv_r[ND];
  double gv_i[ND];

  for(int s = 0; s < ND; ++s){
    id[s] = gm.index(s);
    gv_r[s] = -gm.value_i(s);
    gv_i[s] =  gm.value_r(s);
    idc_r[s] = 1-gm.index_c(s);
    idc_i[s] = 1-idc_r[s];
  }

  int nth = ThreadManager_OpenMP::get_num_threads();
  int ith = ThreadManager_OpenMP::get_thread_id();

  int is  = m_Nvol * ith / nth;
  int ns  = m_Nvol * (ith+1) / nth - is;

  double* v = ptr(m_Nin*is);
  double* w = const_cast<Field_F*>(&x)->ptr(m_Nin*is);

  for(int ex = 0; ex < m_Nex; ++ex){
   for(int site = 0; site < ns; ++site){
     int iv = NCD2 * (site + m_Nvol*ex);
    for(int s = 0; s < ND; ++s){
       int iv2 =    s  * NC2 + iv;
       int iw2 = id[s] * NC2 + iv;
     for(int ic = 0; ic < NC; ++ic){
       v[2*ic   + iv2] = gv_r[s] * w[2*ic+idc_r[s]+iw2];
       v[2*ic+1 + iv2] = gv_i[s] * w[2*ic+idc_i[s]+iw2];
     }
    }
   }
  }

}

//====================================================================
/*
void Field_F::mult_GMproj(const int pm, const GammaMatrix& gm,
                          const Field_F& w)
{
  assert(w.nvol() == m_Nvol);
  assert(w.nex() == m_Nex);

  //  int nth = ThreadManager_OpenMP::get_num_threads();
  //  int ith = ThreadManager_OpenMP::get_thread_id();

  //  int is  = m_Nvol * ith / nth;
  //  int ns  = m_Nvol * (ith+1) / nth - is;

  //  double* x = ptr(is);
  //  double* w = const_cast<Field_F*>(&x)->ptr(is);

  Field_F v(w.nvol(),w.nex());
  v.mult_GM(gm,w);

  int size = m_Nin * m_Nvol * m_Nex;
  if(pm == 1){
    for(int i = 0; i < size; ++i){
      field[i] = 0.5*(w.field[i] + v.field[i]);
    }
  }else if(pm == -1){
    for(int i = 0; i < size; ++i){
      field[i] = 0.5*(w.field[i] - v.field[i]);
    }
  }else{
    vout.general(m_vl, " wrong pm in field_F.cpp.\n");
    abort();
  }

}

//====================================================================
void Field_F::mult_GMproj2(const int pm, const GammaMatrix& gm,
                           const Field_F& w)
{
  assert(w.nvol() == m_Nvol);
  assert(w.nex() == m_Nex);

  Field_F v(w.nvol(),w.nex());
  v.mult_GM(gm,w);

  int size = m_Nin * m_Nvol * m_Nex;
  if(pm == 1){
    for(int i = 0; i < size; ++i){
      field[i] = w.field[i] + v.field[i];
    }
  }else if(pm == -1){
    for(int i = 0; i < size; ++i){
      field[i] = w.field[i] - v.field[i];
    }
  }else{
    vout.general(m_vl, " wrong pm in field_F.cpp.\n");
    abort();
  }

}
*/
//====================================================================
void Field_F::mult_Field_Gn(int ex, const Field_G& U, int ex1,
                            const Field_F& x, int ex2)
{
  assert(ex <= m_Nex);
  assert(ex1 <= U.nex());
  assert(ex2 <= x.nex());
  assert(U.nvol() == m_Nvol);
  assert(x.nvol() == m_Nvol);

  int nth = ThreadManager_OpenMP::get_num_threads();
  int ith = ThreadManager_OpenMP::get_thread_id();

  int is  = m_Nvol * ith / nth;
  int ns  = m_Nvol * (ith+1) / nth - is;

  double* v = ptr(m_Nin*(is + m_Nvol*ex));
  double* g = const_cast<Field_G*>(&U)->ptr(U.nin()*(is + m_Nvol*ex1));
  double* w = const_cast<Field_F*>(&x)->ptr(m_Nin*(is + m_Nvol*ex2));

  for(int site = 0; site < ns; ++site){
    int ig = NDF  * site;
    int iv = NCD2 * site;
   for(int s = 0; s < ND; ++s){
    for(int ic = 0; ic < NC; ++ic){
      int ig2 = ic * NC2 + ig;
      int iv2 = s  * NC2 + iv;
      v[2*ic   + iv2] = mult_Gn_r(&g[ig2], &w[iv2], m_Nc);
      v[2*ic+1 + iv2] = mult_Gn_i(&g[ig2], &w[iv2], m_Nc);
    }
   }
  }

}

//====================================================================
void Field_F::mult_Field_Gd(int ex, const Field_G& U, int ex1,
                            const Field_F& x, int ex2)
{
  assert(ex <= m_Nex);
  assert(ex1 <= U.nex());
  assert(ex2 <= x.nex());
  assert(U.nvol() == m_Nvol);
  assert(x.nvol() == m_Nvol);

  int nth = ThreadManager_OpenMP::get_num_threads();
  int ith = ThreadManager_OpenMP::get_thread_id();

  int is  = m_Nvol * ith / nth;
  int ns  = m_Nvol * (ith+1) / nth - is;

  double* v = ptr(m_Nin*(is + m_Nvol*ex));
  double* g = const_cast<Field_G*>(&U)->ptr(U.nin()*(is + m_Nvol*ex1));
  double* w = const_cast<Field_F*>(&x)->ptr(m_Nin*(is + m_Nvol*ex2));

  for(int site = 0; site < ns; ++site){
    int ig = NDF  * site;
    int iv = NCD2 * site;
   for(int s = 0; s < ND; ++s){
    for(int ic = 0; ic < NC; ++ic){
      int ig2 = ic * 2 + ig;
      int iv2 = s  * NC2 + iv;
      v[2*ic   + iv2] = mult_Gd_r(&g[ig2], &w[iv2], m_Nc);
      v[2*ic+1 + iv2] = mult_Gd_i(&g[ig2], &w[iv2], m_Nc);
    }
   }
  }

}

//====================================================================
void Field_F::multadd_Field_Gn(int ex, const Field_G& U, int ex1,
                               const Field_F& x, int ex2, double a)
{
  assert(ex <= m_Nex);
  assert(ex1 <= U.nex());
  assert(ex2 <= x.nex());
  assert(U.nvol() == m_Nvol);
  assert(x.nvol() == m_Nvol);

  int nth = ThreadManager_OpenMP::get_num_threads();
  int ith = ThreadManager_OpenMP::get_thread_id();

  int is  = m_Nvol * ith / nth;
  int ns  = m_Nvol * (ith+1) / nth - is;

  double* v = ptr(m_Nin*(is + m_Nvol*ex));
  double* g = const_cast<Field_G*>(&U)->ptr(U.nin()*(is + m_Nvol*ex1));
  double* w = const_cast<Field_F*>(&x)->ptr(m_Nin*(is + m_Nvol*ex2));

  for(int site = 0; site < ns; ++site){
    int ig = NDF  * site;
    int iv = NCD2 * site;
   for(int s = 0; s < ND; ++s){
    for(int ic = 0; ic < NC; ++ic){
      int ig2 = ic * NC2 + ig;
      int iv2 = s  * NC2 + iv;
      v[2*ic   + iv2] += a* mult_Gn_r(&g[ig2], &w[iv2], m_Nc);
      v[2*ic+1 + iv2] += a* mult_Gn_i(&g[ig2], &w[iv2], m_Nc);
    }
   }
  }

}

//====================================================================
void Field_F::multadd_Field_Gd(int ex, const Field_G& U, int ex1,
                               const Field_F& x, int ex2, double a)
{
  assert(ex <= m_Nex);
  assert(ex1 <= U.nex());
  assert(ex2 <= x.nex());
  assert(U.nvol() == m_Nvol);
  assert(x.nvol() == m_Nvol);

  int nth = ThreadManager_OpenMP::get_num_threads();
  int ith = ThreadManager_OpenMP::get_thread_id();

  int is  = m_Nvol * ith / nth;
  int ns  = m_Nvol * (ith+1) / nth - is;

  double* v = ptr(m_Nin*(is + m_Nvol*ex));
  double* g = const_cast<Field_G*>(&U)->ptr(U.nin()*(is + m_Nvol*ex1));
  double* w = const_cast<Field_F*>(&x)->ptr(m_Nin*(is + m_Nvol*ex2));

  for(int site = 0; site < ns; ++site){
    int ig = NDF  * site;
    int iv = NCD2 * site;
   for(int s = 0; s < ND; ++s){
    for(int ic = 0; ic < NC; ++ic){
      int ig2 = ic * 2   + ig;
      int iv2 = s  * NC2 + iv;
      v[2*ic   + iv2] += a* mult_Gd_r(&g[ig2], &w[iv2], m_Nc);
      v[2*ic+1 + iv2] += a* mult_Gd_i(&g[ig2], &w[iv2], m_Nc);
    }
   }
  }

}

//====================================================================
//============================================================END=====
