/*!
        @file    $Id:: shiftField_lex_imp.cpp #$

        @brief

        @author  <Hideo Matsufuru> hideo.matsufuru@kek.jp(matsufuru) 
                 $LastChangedBy: matufuru $

        @date    $LastChangedDate:: 2013-10-20 19:28:34 #$

        @version $LastChangedRevision: 972 $
*/


#include "shiftField_lex.h"


//====================================================================
ShiftField_lex::ShiftField_lex(){

  Nx = CommonParameters::Nx();
  Ny = CommonParameters::Ny();
  Nz = CommonParameters::Nz();
  Nt = CommonParameters::Nt();

}                                                                              
//====================================================================
ShiftField_lex::ShiftField_lex(const int Nin){

  Nx = CommonParameters::Nx();
  Ny = CommonParameters::Ny();
  Nz = CommonParameters::Nz();
  Nt = CommonParameters::Nt();
  m_Nin = Nin;

}                                                                              
//====================================================================
ShiftField_lex::~ShiftField_lex(){

}
//====================================================================
void ShiftField_lex::backward(Field& v, const Field& w,
                     const int mu){

  int bc = 1;
  backward(v, w, bc, mu);

}

//====================================================================
void ShiftField_lex::forward(Field& v, const Field& w,
                     const int mu){

  int bc = 1;
  forward(v, w, bc, mu);

}

//====================================================================
void ShiftField_lex::backward_ex(Field& v, int ex1,
                                 const Field& w, int ex2,
                                 const int mu){

  int bc = 1;
  backward_ex(v, ex1, w, ex2, bc, mu);

}

//====================================================================
void ShiftField_lex::forward_ex(Field& v, int ex1,
                                const Field& w, int ex2,
                                const int mu){

  int bc = 1;
  forward_ex(v, ex1, w, ex2, bc, mu);

}

//====================================================================
void ShiftField_lex::backward(Field& v, const Field& w,
			      const int bc, const int mu){

  assert(v.nex() <= w.nex());

  if(mu == 0){
    for(int ex1 = 0; ex1 < v.nex(); ++ex1){
      up_x_ex(&v,ex1,&w,ex1,bc);
    }
  }else if(mu == 1){
    for(int ex1 = 0; ex1 < v.nex(); ++ex1){
      up_y_ex(&v,ex1,&w,ex1,bc);
    }
  }else if(mu == 2){
    for(int ex1 = 0; ex1 < v.nex(); ++ex1){
      up_z_ex(&v,ex1,&w,ex1,bc);
    }
  }else if(mu == 3){
    for(int ex1 = 0; ex1 < v.nex(); ++ex1){
      up_t_ex(&v,ex1,&w,ex1,bc);
    }
  }else{
    std::cout << "wrong parameter: ShiftField_lex\n";
    abort();
  }

}

//====================================================================
void ShiftField_lex::forward(Field& v, const Field& w,
                     const int bc, const int mu){

  assert(v.nex() <= w.nex());

  if(mu == 0){
    for(int ex1 = 0; ex1 < v.nex(); ++ex1){
      dn_x_ex(&v,ex1,&w,ex1,bc);
    }
  }else if(mu == 1){
    for(int ex1 = 0; ex1 < v.nex(); ++ex1){
      dn_y_ex(&v,ex1,&w,ex1,bc);
    }
  }else if(mu == 2){
    for(int ex1 = 0; ex1 < v.nex(); ++ex1){
      dn_z_ex(&v,ex1,&w,ex1,bc);
    }
  }else if(mu == 3){
    for(int ex1 = 0; ex1 < v.nex(); ++ex1){
      dn_t_ex(&v,ex1,&w,ex1,bc);
    }
  }else{
      std::cout << "wrong parameter: ShiftField_lex\n";
    abort();
  }
}

//====================================================================
void ShiftField_lex::backward_ex(Field& v, int ex1,
                                 const Field& w, int ex2,
                                 const int bc, const int mu){

  if(mu == 0){
    up_x_ex(&v,ex1,&w,ex2,bc);
  }else if(mu == 1){
    up_y_ex(&v,ex1,&w,ex2,bc);
  }else if(mu == 2){
    up_z_ex(&v,ex1,&w,ex2,bc);
  }else if(mu == 3){
    up_t_ex(&v,ex1,&w,ex2,bc);
  }else{
    std::cout << "wrong parameter: ShiftField_lex\n";
    abort();
  }

}

//====================================================================
void ShiftField_lex::forward_ex(Field& v, int ex1,
                                const Field& w, int ex2,
                                const int bc, const int mu){

  if(mu == 0){
    dn_x_ex(&v,ex1,&w,ex2,bc);
  }else if(mu == 1){
    dn_y_ex(&v,ex1,&w,ex2,bc);
  }else if(mu == 2){
    dn_z_ex(&v,ex1,&w,ex2,bc);
  }else if(mu == 3){
    dn_t_ex(&v,ex1,&w,ex2,bc);
  }else{
      std::cout << "wrong parameter: ShiftField_lex\n";
    abort();
  }

}

//====================================================================
void ShiftField_lex::up_x_ex(Field* v, int ex1,
                             const Field* w, int ex2, const int bc){

  double bc2 = 1.0;
  if(Communicator::ipe(0) == 0) bc2 = bc;

  int Nin = w->nin();
  int Nvol = w->nvol();
  int Nvol2 = Nvol/Nx;

  double* vp = v->ptr(Nin*Nvol*ex1);
  double* wp = const_cast<Field*>(w)->ptr(Nin*Nvol*ex2);

  double wt[Nin*Nvol2];
  double vt[Nin*Nvol2];

  for(int s2 = 0; s2 < Ny*Nz*Nt; s2++){

    // bulk
    for(int x = 0; x < Nx-1; x++){
      int ix = x + Nx*s2;
      int px = ix + 1;
       for(int in = 0; in < Nin; in++){
         vp[in+Nin*ix] = wp[in+Nin*px];
       }
    }

    // boundary (x=Nx-1)
    int px = Nx*s2;
    int ix = Nx-1 + px;
     for(int in = 0; in < Nin; in++){
       wt[in+Nin*s2] = bc2 * wp[in+Nin*px];
     }

  }

  int size = Nin*Nvol2;
  Communicator::exchange(size,vt,wt,0,1,0);

  for(int s2 = 0; s2 < Ny*Nz*Nt; s2++){
    // boundary (x=Nx-1)
    int px = Nx*s2;
    int ix = Nx-1 + px;
    for(int in = 0; in < Nin; in++){
       vp[in+Nin*ix] = vt[in+Nin*s2];
     }
  }

}

//====================================================================
void ShiftField_lex::dn_x_ex(Field* v, int ex1,
                             const Field* w, int ex2, const int bc){

  double bc2 = 1.0;
  if(Communicator::ipe(0) == 0) bc2 = bc;

  int Nin = w->nin();
  int Nvol = w->nvol();
  int Nvol2 = Nvol/Nx;

  double* vp = v->ptr(Nin*Nvol*ex1);
  double* wp = const_cast<Field*>(w)->ptr(Nin*Nvol*ex2);

  double wt[Nin*Nvol2];
  double vt[Nin*Nvol2];

  for(int s2 = 0; s2 < Ny*Nz*Nt; s2++){

    // bulk
    for(int x = 1; x < Nx; x++){
      int ix = x + Nx*s2;
      int px = ix - 1;
       for(int in = 0; in < Nin; in++){
         vp[in+Nin*ix] = wp[in+Nin*px];
       }
    }

    // boundary (x=0)
    int px = Nx-1 + Nx*s2;
       for(int in = 0; in < Nin; in++){
         wt[in+Nin*s2] = wp[in+Nin*px];
       }

  }

  int size = Nin*Nvol2;
  Communicator::exchange(size,vt,wt,0,-1,4);

  for(int s2 = 0; s2 < Ny*Nz*Nt; s2++){
    // boundary (x=0)
    int ix = Nx*s2;
     for(int in = 0; in < Nin; in++){
       vp[in+Nin*ix] = bc2 * vt[in+Nin*s2];
     }
     //    }
  }

}


//====================================================================
void ShiftField_lex::up_y_ex(Field* v, int ex1,
                             const Field* w, int ex2, const int bc){

  double bc2 = 1.0;
  if(Communicator::ipe(1) == 0) bc2 = bc;

  int Nin = w->nin();
  int Nvol = w->nvol();
  int Nvol2 = Nvol/Ny;

  double* vp = v->ptr(Nin*Nvol*ex1);
  double* wp = const_cast<Field*>(w)->ptr(Nin*Nvol*ex2);

  double wt[Nin*Nvol2];
  double vt[Nin*Nvol2];

  for(int zt = 0; zt < Nz*Nt; zt++){
    for(int x = 0; x < Nx; x++){
      int s1 = x + Nx*Ny*zt;
      int s2 = x + Nx*zt;

      // bulk
      for(int y = 0; y < Ny-1; y++){
          int ix = s1 + Nx*y;
          int px = ix + Nx;
          for(int in = 0; in < Nin; in++){
            vp[in+Nin*ix] = wp[in+Nin*px];
          }
      }

      // boundary (y=Ny-1)
      int px = s1;
       for(int in = 0; in < Nin; in++){
        wt[in+Nin*s2] = bc2 * wp[in+Nin*px];
       }

    }
  }

  int size = Nin*Nvol2;
  Communicator::exchange(size,vt,wt,1,1,1);

  for(int zt = 0; zt < Nz*Nt; zt++){
    for(int x = 0; x < Nx; x++){
      int s1 = x + Nx*Ny*zt;
      int s2 = x + Nx*zt;

      // boundary (y=Ny-1)
        int ix = s1 + Nx*(Ny-1);
         for(int in = 0; in < Nin; in++){
           vp[in+Nin*ix] = vt[in+Nin*s2];
          }

    }
  }

}

//====================================================================
void ShiftField_lex::dn_y_ex(Field* v, int ex1,
                             const Field* w, int ex2, const int bc){

  double bc2 = 1.0;
  if(Communicator::ipe(1) == 0) bc2 = bc;

  int Nin = w->nin();
  int Nvol = w->nvol();
  int Nvol2 = Nvol/Ny;

  double* vp = v->ptr(Nin*Nvol*ex1);
  double* wp = const_cast<Field*>(w)->ptr(Nin*Nvol*ex2);

  double wt[Nin*Nvol2];
  double vt[Nin*Nvol2];

  for(int zt = 0; zt < Nz*Nt; zt++){
    for(int x = 0; x < Nx; x++){
      int s1 = x + Nx*Ny*zt;
      int s2 = x + Nx*zt;

      // bulk
      for(int y = 1; y < Ny; y++){
          int ix = s1 + Nx*y;
          int px = ix - Nx;
          for(int in = 0; in < Nin; in++){
            vp[in+Nin*ix] = wp[in+Nin*px];
          }
      }

      // boundary (y=0)
      int px = s1 + Nx*(Ny-1);
       for(int in = 0; in < Nin; in++){
         wt[in+Nin*s2] = wp[in+Nin*px];
       }

    }
  }

  int size = Nin*Nvol2;
  Communicator::exchange(size,vt,wt,1,-1,5);

  for(int zt = 0; zt < Nz*Nt; zt++){
    for(int x = 0; x < Nx; x++){
      int s1 = x + Nx*Ny*zt;
      int s2 = x + Nx*zt;

      // boundary (y=0)
      int ix = s1;
       for(int in = 0; in < Nin; in++){
         vp[in+Nin*ix] = bc2 * vt[in+Nin*s2];
       }

    }
  }

}


//====================================================================
void ShiftField_lex::up_z_ex(Field* v, int ex1,
                             const Field* w, int ex2, const int bc){

  double bc2 = 1.0;
  if(Communicator::ipe(2) == 0) bc2 = bc;

  int Nin = w->nin();
  int Nvol = w->nvol();
  int Nvol2 = Nvol/Nz;

  double* vp = v->ptr(Nin*Nvol*ex1);
  double* wp = const_cast<Field*>(w)->ptr(Nin*Nvol*ex2);

  double wt[Nin*Nvol2];
  double vt[Nin*Nvol2];

  for(int t = 0; t < Nt; t++){
   for(int xy = 0; xy < Nx*Ny; xy++){
     int s1 = xy + Nx*Ny*Nz*t;
     int s2 = xy + Nx*Ny*t;

     // bulk
     for(int z = 0; z < Nz-1; z++){
       int ix = s1 + Nx*Ny*z;
       int px = ix + Nx*Ny;
        for(int in = 0; in < Nin; in++){
          vp[in+Nin*ix] = wp[in+Nin*px];
        }
     }

     // boundary (z=Nz-1)
     int px = s1;
      for(int in = 0; in < Nin; in++){
        wt[in+Nin*s2] = bc2 * wp[in+Nin*px];
      }

   }
  }

  int size = Nin*(Nvol/Nz);
  Communicator::exchange(size,vt,wt,2,1,2);

  for(int t = 0; t < Nt; t++){
   for(int xy = 0; xy < Nx*Ny; xy++){
     int s1 = xy + Nx*Ny*Nz*t;
     int s2 = xy + Nx*Ny*t;

     // boundary (z=Nz-1)
     int ix = s1 + Nx*Ny*(Nz-1);
      for(int in = 0; in < Nin; in++){
        vp[in+Nin*ix] = vt[in+Nin*s2];
      }

   }
  }

}

//====================================================================
void ShiftField_lex::dn_z_ex(Field* v, int ex1,
                             const Field* w, int ex2, const int bc){

  double bc2 = 1.0;
  if(Communicator::ipe(2) == 0) bc2 = bc;

  int Nin = w->nin();
  int Nvol = w->nvol();
  int Nvol2 = Nvol/Nz;

  double* vp = v->ptr(Nin*Nvol*ex1);
  double* wp = const_cast<Field*>(w)->ptr(Nin*Nvol*ex2);

  double wt[Nin*Nvol2];
  double vt[Nin*Nvol2];

  for(int t = 0; t < Nt; t++){
   for(int xy = 0; xy < Nx*Ny; xy++){
     int s1 = xy + Nx*Ny*Nz*t;
     int s2 = xy + Nx*Ny*t;

     // bulk
     for(int z = 1; z < Nz; z++){
       int ix = s1 + Nx*Ny*z;
       int px = ix - Nx*Ny;
        for(int in = 0; in < Nin; in++){
          vp[in+Nin*ix] = wp[in+Nin*px];
        }
     }
     // boundary (z=0)
     int px = s1 + Nx*Ny*(Nz-1);
     for(int in = 0; in < Nin; in++){
        wt[in+Nin*s2] = wp[in+Nin*px];
     }

   }
  }

  int size = Nin*Nvol2;
  Communicator::exchange(size,vt,wt,2,-1,6);

  for(int t = 0; t < Nt; t++){
   for(int xy = 0; xy < Nx*Ny; xy++){
     int s1 = xy + Nx*Ny*Nz*t;
     int s2 = xy + Nx*Ny*t;

     // boundary (z=0)
     int ix = s1;
     for(int in = 0; in < Nin; in++){
        vp[in+Nin*ix] = bc2 * vt[in+Nin*s2];
     }

   }
  }

}


//====================================================================
void ShiftField_lex::up_t_ex(Field* v, int ex1,
                             const Field* w, int ex2, const int bc){

  double bc2 = 1.0;
  if(Communicator::ipe(3) == 0) bc2 = bc;

  int Nin = w->nin();
  int Nvol = w->nvol();
  int Nvol2 = Nvol/Nt;

  double* vp = v->ptr(Nin*Nvol*ex1);
  double* wp = const_cast<Field*>(w)->ptr(Nin*Nvol*ex2);

  double wt[Nin*Nvol2];
  double vt[Nin*Nvol2];

  for(int s2 = 0; s2 < Nx*Ny*Nz; s2++){

    // bulk
    for(int t = 0; t < Nt-1; t++){
      int ix = s2 + Nx*Ny*Nz*t;
      int px = ix + Nx*Ny*Nz;
       for(int in = 0; in < Nin; in++){
         vp[in+Nin*ix] = wp[in+Nin*px];
       }
    }

    // boundary (t=Nt-1)
    int px = s2;
     for(int in = 0; in < Nin; in++){
       wt[in+Nin*s2] = bc2 * wp[in+Nin*px];
     }
  }

  int size = Nin*Nvol2;
  Communicator::exchange(size,vt,wt,3,1,3);

  for(int s2 = 0; s2 < Nx*Ny*Nz; s2++){
    // boundary (t=Nt-1)
    int ix = s2 + Nx*Ny*Nz*(Nt-1);
     for(int in = 0; in < Nin; in++){
       vp[in+Nin*ix] = vt[in+Nin*s2];
     }
  }

}

//====================================================================
void ShiftField_lex::dn_t_ex(Field* v, int ex1,
                             const Field* w, int ex2, const int bc){

  double bc2 = 1.0;
  if(Communicator::ipe(3) == 0) bc2 = bc;

  int Nin = w->nin();
  int Nvol = w->nvol();
  int Nvol2 = Nvol/Nt;

  double* vp = v->ptr(Nin*Nvol*ex1);
  double* wp = const_cast<Field*>(w)->ptr(Nin*Nvol*ex2);

  double wt[Nin*Nvol2];
  double vt[Nin*Nvol2];

  for(int s2 = 0; s2 < Nx*Ny*Nz; s2++){

    // bulk
    for(int t = 1; t < Nt; t++){
      int ix = s2 + Nx*Ny*Nz*t;
      int px = ix - Nx*Ny*Nz;
       for(int in = 0; in < Nin; in++){
         vp[in+Nin*ix] = wp[in+Nin*px];
       }
    }
    // boundary (t=0)
    int px = s2 + Nx*Ny*Nz*(Nt-1);
     for(int in = 0; in < Nin; in++){
       wt[in+Nin*s2] = wp[in+Nin*px];
     }

  }

  int size = Nin*Nvol2;
  Communicator::exchange(size,vt,wt,3,-1,7);

  for(int s2 = 0; s2 < Nx*Ny*Nz; s2++){

    // boundary (t=0)
    int ix = s2;
     for(int in = 0; in < Nin; in++){
       vp[in+Nin*ix] = bc2 * vt[in+Nin*s2];
     }

  }

}

//====================================================================
//============================================================END=====
