/*!
        @file    $Id:: staples.h #$

        @brief

        @author  <Hideo Matsufuru> hideo.matsufuru@kek.jp(matsufuru) 
                 $LastChangedBy: matufuru $

        @date    $LastChangedDate:: 2014-06-02 23:54:18 #$

        @version $LastChangedRevision: 1054 $
*/

#ifndef STAPLES_INCLUDED
#define STAPLES_INCLUDED

#include "defs.h"
#include "field_G.h"

#include "bridgeIO.h"

class ShiftField_lex;

//! Staple construction.
/*!
    This class constructs the staples.
    While the originial version was written by J.Noaki,
    the present version is completely modified by H.Matsufuru
    except for the interface.
                               [28 Dec 2011 H.Matsufuru]
    Thread-parallelized.
    void version of upper and lower functions added; these are
    faster than the versions returning Field_G object.
                               [28 Sep 2013 H.Matsufuru]
 */

class Staples
{
 protected:
  Bridge::VerboseLevel m_vl;

 private:
  ShiftField_lex* m_shift;
  Field_G m_staple, m_v, m_w;      //!< temporary fields.

 public:

  Staples();
  ~Staples();

  //! sets verbose level.
  void set_parameter_verboselevel(const Bridge::VerboseLevel vl) { m_vl = vl; }

  //! constructs upper staple in mu-nu plane (wrapping void version).
  Field_G upper(const Field_G&, const int mu, const int nu);

  //! constructs lower staple in mu-nu plane (wrapping void version).
  Field_G lower(const Field_G&, const int mu, const int nu);

  //! constructs upper staple in mu-nu plane.
  void upper(Field_G&, const Field_G&, const int mu, const int nu);

  //! constructs lower staple in mu-nu plane.
  void lower(Field_G&, const Field_G&, const int mu, const int nu);

  //! constructs staples in mu-direction (summing up nu-direction).
  void staple(Field_G&, const Field_G&, const int mu);

  //! calculates plaquette value.
  double plaquette(const Field_G&);
  
  //! calculates spatial plaquette value.
  double plaq_s(const Field_G&);

  //! calculates temporal plaquette value.
  double plaq_t(const Field_G&);
  
};

#endif  
