/*!
        @file    $Id:: threadManager_OpenMP.h #$

        @brief

        @author  <Hideo Matsufuru> hideo.matsufuru@kek.jp(matsufuru) 
                 $LastChangedBy: matufuru $

        @date    $LastChangedDate:: 2013-01-04 14:58:10 #$

        @version $LastChangedRevision: 819 $
*/

#ifndef THREADMANAGER_OPENMP_INCLUDED
#define THREADMANAGER_OPENMP_INCLUDED

//#include <pthread.h>
#include <valarray>

//#include "configure.h"
//#include "defs.h"
#include "commonParameters.h"
#include "bridgeIO.h"


//! Thread manager with OpenMP.
/*!
   This class wraps OpenMP API functions.
   The number of available threads is specified as an argument
   of init() function, which is to be called at the begining of
   main, soon after Communicator and CommonParameter are
   initialized.   
   If the specified number of threads (nthread) is not the same
   as that of OMP_NUM_THREADS environment variable, the number
   of threads is set to the specified one.
   If nthread = 0 is specified, the number of threads is set
   to OMP_NUM_THREADS or maximum number of platform.
                                      [29 Aug 2013 H.Matsufuru]
 */
class ThreadManager_OpenMP{

 private:
  static int m_Nthread;              //!< number of threads.
  static Bridge::VerboseLevel m_vl;  //!< verbose level.
  static valarray<double> m_darray_reduction;

 public:

  //! setup: called in main only once.
  static void init(int nthread);

  //! finalization.
  static void finalize();

  //! returns number of threads (works outside of parallel region).
  static int get_num_threads_available(){ return m_Nthread; };

  //! returns available number of threads.
  static int get_num_threads();

  //! returns thread id.
  static int get_thread_id();

  //! barrier among threads inside a node.
  static void wait();

  //! barrier among threads inside a node.
  static void barrier(const int nth);

  //! barrier among all the threads and nodes.
  static void sync_barrier_all();

  //! global reduction with summation: value is assumed thread local.
  static void reduce_sum_global(double& value,
                                const int ith, const int nth);

};

#endif //THREADMANAGER_OPENMP_INCLUDED
